//  Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.hivemind.service.impl;

import java.util.HashMap;
import java.util.Map;

import javassist.CtClass;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.internal.Module;
import org.apache.hivemind.service.ClassFab;
import org.apache.hivemind.service.ClassFactory;

/**
 * Implementation of {@link org.apache.hivemind.service.ClassFactory}.
 *
 * @author Howard Lewis Ship
 */
public class ClassFactoryImpl implements ClassFactory
{
    /**
     * Map of CtClassSource, keyed on module id.
     */
    private Map _ctClassSourceMap = new HashMap();

    public ClassFab newClass(String name, Class superClass, Module module)
    {
        CtClassSource source = findCtClassSource(module);

        try
        {
            CtClass ctNewClass = source.newClass(name, superClass);

            return new ClassFabImpl(source, ctNewClass);
        }
        catch (Exception ex)
        {
            throw new ApplicationRuntimeException(
                ServiceMessages.unableToCreateClass(name, superClass, ex),
                ex);
        }

    }

    private synchronized CtClassSource findCtClassSource(Module module)
    {
        String id = module.getModuleId();

        CtClassSource result = (CtClassSource) _ctClassSourceMap.get(id);

        if (result == null)
        {
            result = new CtClassSource(module.getClassResolver().getClassLoader());

            _ctClassSourceMap.put(id, result);
        }

        return result;
    }

}
